/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * MoleculeIdDescriptor.cpp
 *
 *  Created on: Feb 5, 2010
 *      Author: crueger
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "MoleculeIdDescriptor.hpp"
#include "MoleculeIdDescriptor_impl.hpp"

#include "CodePatterns/Observer/ObservedContainer_impl.hpp"

#include "molecule.hpp"

using namespace std;


MoleculeIdDescriptor_impl::MoleculeIdDescriptor_impl(moleculeId_t _id) :
  id(_id)
{}

MoleculeIdDescriptor_impl::~MoleculeIdDescriptor_impl()
{}

bool MoleculeIdDescriptor_impl::predicate(const std::pair<moleculeId_t,const molecule*> molecule)  const{
  return molecule.first==id;
}

MoleculeDescriptor MoleculeById(moleculeId_t id){
  return MoleculeDescriptor(MoleculeDescriptor::impl_ptr(new MoleculeIdDescriptor_impl(id)));
}

molecule *MoleculeIdDescriptor_impl::find(){
  World::MoleculeSet &molecules = getMolecules();
  World::MoleculeSet::iterator res = molecules.find(id);
  return (res!=molecules.end())?((*res).second):0;
}

const molecule *MoleculeIdDescriptor_impl::find() const {
  const World::MoleculeSet &molecules = getMolecules();
  const World::MoleculeSet::const_iterator res = molecules.find(id);
  return (res!=molecules.end())?((*res).second):0;
}

vector<molecule*> MoleculeIdDescriptor_impl::findAll(){
  molecule *res = find();
  return (res)?(vector<molecule*>(1,res)):(vector<molecule*>());
}

vector<const molecule*> MoleculeIdDescriptor_impl::findAll() const {
  const molecule *res = find();
  return (res)?(vector<const molecule*>(1,res)):(vector<const molecule*>());
}

